import peasy.*;       //PeasyCamライブラリをインストールしてインポート
import java.util.Date;   //Dateオブジェクトを使用するため、javaのパッケージをインポート
import controlP5.*;    //スライダーを使用するため、Control P5ライブラリをインポート

PeasyCam cam;
Table table;
PImage japanmap;
PFont font;

//経度の範囲
float lonFrom = 122.0;
float lonTo = 148.0;

//緯度の範囲
float latFrom = 24.0;
float latTo = 46.0;

//時間の範囲（単位は秒、ただしUnixtime）
int timeBegin = 1571721423;
int timeEnd =   1571727741;
int timeNow;

//通過した最後のポイント
int point;

//ControlP5オブジェクトの宣言
ControlP5 cp5;
//スライダーオブジェクトの宣言
Slider timeSlider; 

//オートプレイ中かどうか
boolean isAutoPlay = false;

//アイコン用画像
PImage icon;

void setup() {
  size(1024, 768, P3D);
  cam = new PeasyCam(this, 700);

  //CSVファイルをTableに読み込む
  table = loadTable("JL321_229092b2.csv", "header");

  //地図画像を読み込む
  japanmap = loadImage("map-japan.png");

  //アイコン画像を読み込む
  icon = loadImage("icon_aircraft.png");

  colorMode(HSB);
  font = createFont("Osaka", 12);
  textFont(font);

  //開始時刻をtimeBeginに合わせる
  timeNow = timeBegin;

  //ControlP5のオブジェクトを生成
  cp5 = new ControlP5(this);
  cp5.setAutoDraw(false);

  //スライダーのラベル、バーの色、値の範囲、初期の値、表示位置、表示サイズを指定
  timeSlider = cp5.addSlider("timeSlider");
  timeSlider.setFont(font);
  timeSlider.setLabel("Flight Time (sec)");
  timeSlider.setColorForeground( color(30, 255, 190) );
  timeSlider.setColorBackground( color(0, 0, 90) );
  timeSlider.setColorActive( color(40, 255, 190) );
  timeSlider.setRange(0, timeEnd-timeBegin);
  timeSlider.setValue(0);
  timeSlider.setPosition(0, height-30);
  timeSlider.setSize(width-120, 30);
}

void draw() {
  background(0);
  translate(-japanmap.width/2, -height/2);

  //地図の描画
  noStroke();
  beginShape();

  tint(0, 0, 255);
  texture(japanmap);
  vertex(0, 0, 0, 0, 0);//vertex(x,y,z,textureのu,textureのv)
  vertex(japanmap.width, 0, 0, japanmap.width, 0);
  vertex(japanmap.width, japanmap.height, 0, japanmap.width, japanmap.height);
  vertex(0, japanmap.height, 0, 0, japanmap.height);
  endShape();

  //データから全ての行を取り出す

  noFill();
  strokeWeight(1);
  beginShape();

  for (int i=0; i<table.getRowCount(); i++) {
    float lat = table.getRow(i).getFloat("Lat");
    float lon = table.getRow(i).getFloat("Lon");
    float alt = table.getRow(i).getFloat("Altitude");
    float speed = table.getRow(i).getFloat("Speed");

    float x = map(lon, lonFrom, lonTo, 0, japanmap.width);
    float y = map(lat, latFrom, latTo, japanmap.height, 0);
    float z = map(alt, 0, 40000, 0, 60);
    float hue = map(speed, 0, 500, 192, 0);
    stroke(hue, 255, 255);
    vertex(x, y, z);
  }

  endShape();

  fill(0, 0, 255);
  noStroke();
  //データのある位置をプロットする
  for (int i=0; i<table.getRowCount(); i++) {
    float lat = table.getRow(i).getFloat("Lat");
    float lon = table.getRow(i).getFloat("Lon");
    float alt = table.getRow(i).getFloat("Altitude");

    float x = map(lon, lonFrom, lonTo, 0, japanmap.width);
    float y = map(lat, latFrom, latTo, japanmap.height, 0);
    float z = map(alt, 0, 40000, 0, 60);

    //すでに通過済みのポイントは赤くする
    int time = table.getRow(i).getInt("Timestamp");
    if (time < timeNow) {
      fill(0, 255, 255);
    } else {
      fill(255, 0, 255);
    }

    pushMatrix();
    translate(x, y, z);
    sphere(0.5);
    popMatrix();

    //ポイント10個毎に通過時刻を表示
    if (i%10 ==0) {
      Date date = new Date((long)time*1000);      //Unix TimeからDateオブジェクトに
      String timeString = date.toLocaleString();  //Dateオブジェクトから日付文字列に

      pushMatrix();
      fill(0, 0, 255);
      translate(x, y, z);
      rotateX( radians(-90) );
      textSize(2);
      text(i, 0, -3, 0); //通過ポイントの番号の表示
      text(timeString, 0, 0, 0);  //通過時刻の表示
      popMatrix();
    }
  }

  //今どのポイントにいるか特定する
  point = wherePoint(timeNow);

  //特定されたポイントから位置を計算
  float latNow = table.getRow(point).getFloat("Lat");
  float lonNow = table.getRow(point).getFloat("Lon");
  float altNow = table.getRow(point).getFloat("Altitude");
  float dirNow = table.getRow(point).getFloat("Direction"); //進行方向の角度（北=0 南=180)

  float xNow = map(lonNow, lonFrom, lonTo, 0, japanmap.width);
  float yNow = map(latNow, latFrom, latTo, japanmap.height, 0);
  float zNow = map(altNow, 0, 40000, 0, 60);

  //特定されたポイントの「次のポイント」の位置を計算
  float latNext = table.getRow(point+1).getFloat("Lat");
  float lonNext = table.getRow(point+1).getFloat("Lon");
  float altNext = table.getRow(point+1).getFloat("Altitude");
  float dirNext = table.getRow(point+1).getFloat("Direction");

  float xNext = map(lonNext, lonFrom, lonTo, 0, japanmap.width);
  float yNext = map(latNext, latFrom, latTo, japanmap.height, 0);
  float zNext = map(altNext, 0, 40000, 0, 60);

  //次のポイントを通過する時間から、今のポイントを通過した時間を差し引いて、
  //ポイント間の移動にかかる時間を計算（秒）
  int dulation = table.getRow(point+1).getInt("Timestamp")
    - table.getRow(point).getInt("Timestamp");

  //現在の時間から、今のポイントを通過した時間を差し引いて、
  //最後のポイントを通過してから経過した時間を計算（秒）
  int passTime = timeNow - table.getRow(point).getInt("Timestamp");

  //passTimeをdulationで割れば、次のポイントまで何%進んでいるかわかる。
  //%をもとにlerp関数で補間した現在地を計算する。
  float percent = float(passTime) / float(dulation);
  float xLerp = lerp(xNow, xNext, percent);
  float yLerp = lerp(yNow, yNext, percent);
  float zLerp = lerp(zNow, zNext, percent);
  float altLerp = lerp(altNow, altNext, percent);
  float dirLerp = lerp(dirNow, dirNext, percent);

  //補間した情報で現在地をプロットする
  pushMatrix();
  translate(xLerp, yLerp, zLerp);
  rotateZ( radians(dirLerp) ); //z軸を中心に進行方向に回転
  tint(150, 255, 255);  //アイコンは青に着色
  image(icon, -10, -10, 20, 20);  //縦横20pxで、アイコンを表示
  popMatrix();

  //オートプレイ中と手動操作中の処理の違い
  if (isAutoPlay) {
    //時間の進行
    timeNow += 1;
    if (timeNow > timeEnd) {
      timeNow = timeBegin;
    }
    timeSlider.setValue(timeNow-timeBegin);
  } else {
    //スライダー指定した時間を取得する
    timeNow = int(timeSlider.getValue()) + timeBegin;
  }

  //画面左上の時間の表示  
  Date date = new Date((long)timeNow*1000);      //Unix TimeからDateオブジェクトに
  String timeNowString = date.toLocaleString();  //Dateオブジェクトから日付文字列に
  fill(0, 0, 255);
  cam.beginHUD();
  textSize(12);
  text(timeNow, 20, 20, 0);
  text(timeNowString, 20, 40, 0);
  text(point, 20, 60, 0);
  text("press Space key to Play or Pause", 200, 20, 0);
  cam.endHUD();

  //補間した位置や高度なども表示してみる
  fill(0, 0, 255);
  stroke(0, 0, 255);


  pushMatrix();
  translate(xLerp, yLerp, zLerp);
  rotateX( radians(-90) );
  String callSign = table.getRow(point).getString("Callsign");
  float lonLerp = lerp(lonNow, lonNext, percent);
  float latLerp = lerp(latNow, latNext, percent);
  textSize(8);
  line(0, 0, 0, 0, -100, 0);
  text(callSign, 10, -90, 0);
  text("time "+timeNowString, 10, -80, 0); 
  text("lon: "+lonLerp, 10, -70, 0);
  text("lat: "+latLerp, 10, -60, 0);
  popMatrix();

  //スライダーの表示
  cam.beginHUD();
  cp5.draw();
  cam.endHUD();

  //スライダーをドラッグしているときはcamが動かないようにする
  //スライダーをドラッグしたら、オートプレイモードを抜ける
  if (cp5.isMouseOver() && cam.isActive()) {
    cam.setActive(false);
    isAutoPlay = false;
  } else if ( !cp5.isMouseOver() && !cam.isActive() ) {
    cam.setActive(true);
  }
}

void keyPressed() {
  if (key ==' ') {
    if (isAutoPlay) {
      isAutoPlay = false;
    } else {
      isAutoPlay = true;
    }
  }
}

int wherePoint(int timeNow) {
  int point = 0;
  for (int i=0; i<table.getRowCount(); i++) {
    int time = table.getRow(i).getInt("Timestamp");
    if (time < timeNow) {
      point = i;
    }
  }
  return point;
}
