import peasy.*; //PeasyCamライブラリをインストールしてインポート
PeasyCam cam;
Table table;
PImage japanmap;
PFont font;

//経度の範囲
float lonFrom = 122.0;
float lonTo = 148.0;

//緯度の範囲
float latFrom = 24.0;
float latTo = 46.0;

//時間の範囲（単位は秒、ただしUnixtime）
int timeBegin = 1571721423;
int timeEnd =   1571727741;
int timeNow;

void setup() {
  size(1024, 768, P3D);

  //CSVファイルをTableに読み込む
  table = loadTable("JL321_229092b2.csv", "header");

  //地図画像を読み込む
  japanmap = loadImage("map-japan.png");

  cam = new PeasyCam(this, 700);

  colorMode(HSB);
  font = createFont("Osaka", 12);
  textFont(font);
  
  //開始時刻をtimeBeginに合わせる
  timeNow = timeBegin;
}


void draw() {
  background(0);
  translate(-japanmap.width/2, -height/2);

  //地図の描画
  noStroke();
  beginShape();
  texture(japanmap);
  vertex(0, 0, 0, 0, 0);//vertex(x,y,z,textureのu,textureのv)
  vertex(japanmap.width, 0, 0, japanmap.width, 0);
  vertex(japanmap.width, japanmap.height, 0, japanmap.width, japanmap.height);
  vertex(0, japanmap.height, 0, 0, japanmap.height);
  endShape();

  //データから全ての行を取り出す

  noFill();
  strokeWeight(1);
  beginShape();

  for (int i=0; i<table.getRowCount(); i++) {
    float lat = table.getRow(i).getFloat("Lat");
    float lon = table.getRow(i).getFloat("Lon");
    float alt = table.getRow(i).getFloat("Altitude");
    float speed = table.getRow(i).getFloat("Speed");

    float x = map(lon, lonFrom, lonTo, 0, japanmap.width);
    float y = map(lat, latFrom, latTo, japanmap.height, 0);
    float z = map(alt, 0, 40000, 0, 60);
    float hue = map(speed, 0, 500, 192, 0);
    stroke(hue, 255, 255);
    vertex(x, y, z);
  }

  endShape();
  
  fill(0,0,255);
  noStroke();
  //データのある位置をプロットする
  for (int i=0; i<table.getRowCount(); i++) {
    float lat = table.getRow(i).getFloat("Lat");
    float lon = table.getRow(i).getFloat("Lon");
    float alt = table.getRow(i).getFloat("Altitude");

    float x = map(lon, lonFrom, lonTo, 0, japanmap.width);
    float y = map(lat, latFrom, latTo, japanmap.height, 0);
    float z = map(alt, 0, 40000, 0, 60);
    
    //すでに通過済みのポイントは赤くする
    int time = table.getRow(i).getInt("Timestamp");
    if(time < timeNow){
      fill(0,255,255);
    }
    else{
      fill(255,0,255);
    }
    
    pushMatrix();
    translate(x,y,z);
    sphere(0.5);
    popMatrix();
  }
  
  
  //時間の進行
  timeNow += 1;
  if(timeNow > timeEnd){
    timeNow = timeBegin;
  }
  //時間の表示
  cam.beginHUD();
  text(timeNow,20,20,0);
  cam.endHUD();
  
  
}
